SystemOrganization addCategory: #'GetText-Editor'!

Object subclass: #GetTextExporter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GetText-Editor'!

!GetTextExporter commentStamp: '<historical>' prior: 0!
Export translations to gettext format divided into categories.

"Export gettext template files"
GetTextExporter new exportTemplate.

"Export translation files for current locale"
GetTextExporter new exportTranslator: (InternalTranslator newLocaleID: LocaleID current).

"Export all gettext template and po files."
GetTextExporter exportAll.

!

!GetTextExporter class methodsFor: 'utilities' stamp: 'tak 10/18/2007 21:14'!
coverageStatus
	"self coverageStatus"
	| keys diff |
	keys := self keys.
	diff := InternalTranslator allKnownPhrases keys difference: keys.
	Transcript cr; show: 'Detected keywords by GetTextExporter2: ' , keys size printString.
	Transcript cr; show: 'All known phrases in InternalTranslator: ' , InternalTranslator allKnownPhrases size printString.
	Transcript cr; show: 'Coverage: ' , (keys size / InternalTranslator allKnownPhrases size * 100.0) printString , '%'.
	diff inspect! !

!GetTextExporter class methodsFor: 'utilities' stamp: 'tak 10/18/2007 21:14'!
exportAll
	"GetTextExporter2 exportAll"
	self new exportTemplate.
	InternalTranslator availableLanguageLocaleIDs
		do: [:each | self new exportTranslator: each translator]! !

!GetTextExporter class methodsFor: 'utilities' stamp: 'tak 9/16/2007 14:04'!
exportTemplate
	"GetTextExporter2 exportTemplate"
	self new exportTemplate.! !

!GetTextExporter class methodsFor: 'utilities' stamp: 'Richo 5/25/2010 21:56'!
keys
	| categories |
	categories := Dictionary new.
	self new appendTranslations: categories.
	^ categories values
		inject: Set new
		into: [:set :next | set addAll: next keys;
				 yourself]! !

!GetTextExporter class methodsFor: 'utilities' stamp: 'tak 7/25/2007 13:57'!
listAllHelp
	"self listAllHelp"
	| spec specs oCatalog flap flapSelectors allKeys oCatalogHelp flapHelp |
	oCatalog := Dictionary new.
	Morph withAllSubclasses
		do: [:aClass | (aClass class includesSelector: #descriptionForPartsBin)
				ifTrue: [spec := aClass descriptionForPartsBin.
					oCatalog at: spec formalName put: spec documentation]].
	Morph withAllSubclasses
		do: [:aClass | (aClass class includesSelector: #supplementaryPartsDescriptions)
				ifTrue: [specs := aClass supplementaryPartsDescriptions.
					specs
						do: [:each | oCatalog at: each formalName put: each documentation]]].
	flap := Dictionary new.
	flapSelectors := #(#defaultsQuadsDefiningPlugInSuppliesFlap #defaultsQuadsDefiningStackToolsFlap #defaultsQuadsDefiningSuppliesFlap #defaultsQuadsDefiningToolsFlap #defaultsQuadsDefiningWidgetsFlap #defaultsQuadsDefiningScriptingFlap ).
	flapSelectors
		do: [:selector | 
			specs := Flaps perform: selector.
			specs
				do: [:each | flap at: each third put: each fourth]].
	allKeys := oCatalog keys intersection: flap keys.
	allKeys asArray sort
		do: [:each | 
			oCatalogHelp := oCatalog
						at: each
						ifAbsent: [''].
			flapHelp := flap
						at: each
						ifAbsent: [''].
			oCatalogHelp = flapHelp
				ifFalse: [Transcript cr; show: 'Name: ' , each.
					Transcript cr; show: 'O: ' , oCatalogHelp.
					Transcript cr; show: 'F: ' , flapHelp.
					Transcript cr.

]]! !

!GetTextExporter class methodsFor: 'utilities' stamp: 'Richo 5/25/2010 21:56'!
verifyExport
	"Same as #verifyMsgID: but it writes / reads .po files actually"
	"GetTextExporter2 verifyExport"
	"InternalTranslator removeLocaleID: (LocaleID isoString: 'test-US')"
	| src dst localeID |
	localeID := LocaleID isoString: 'test-US'.
	self verifyMsgID: localeID.
	src := localeID translator.
	self new exportTranslator: src.
	InternalTranslator removeLocaleID: localeID.
	dst := localeID translator.
	GetTextImporter import: dst allDirectory: FileDirectory default! !

!GetTextExporter class methodsFor: 'utilities' stamp: 'tak 10/18/2007 21:14'!
verifyMsgID: localeID 
	"GetTextExporter2 verifyMsgID: (LocaleID isoString: 'test-US')"
	"InternalTranslator removeLocaleID: (LocaleID isoString: 'test-US')"
	"Test gettext keyword extract function without file I/O.
	A language named <langName> will be made. And
	all possible translated words are shown with extra X charactor like
	'XwordX' in the language."
	| src |
	InternalTranslator removeLocaleID: localeID.
	src := localeID translator.
	self keys
		do: [:key | src generics at: key put: 'X' , key , 'X']! !

!GetTextExporter methodsFor: 'private' stamp: 'Richo 5/25/2010 22:05'!
appendStringReceivers: aSymbol into: domains
	| literals references domainName methodReference keywords found |
	
	found := TranslatedReceiverFinder new stringReceiversWithContext: aSymbol.
	found do: [ :assoc |
		methodReference := assoc key.
		keywords := assoc value.
		domainName := (PackageOrganizer default packageOfMethod: methodReference ifNone: [nil]).
		domainName := domainName isNil ifTrue: [TextDomainManager defaultDomain] ifFalse: [domainName name].
		literals := domains at: domainName ifAbsentPut: [Dictionary new].
		keywords do: [ :literal |
			references := literals at: literal ifAbsentPut: [OrderedCollection new].
			references add: methodReference.
		].
	]. 

! !

!GetTextExporter methodsFor: 'exporting' stamp: 'KR 9/28/2007 13:16'!
appendTranslations: domains 
	self appendStringReceivers: #translated into: domains.
	self appendStringReceivers: #translatedNoop into: domains.
	self appendVocabularies: domains.
! !

!GetTextExporter methodsFor: 'private' stamp: 'fbs 5/21/2013 22:42'!
appendVocabularies: domains
	| literalsForDomain references domainName methodReference |
	
	EToyVocabulary allPhrasesWithContextToTranslate do: [ :r |
		methodReference :=  (MethodReference class: (r second) selector: (r third)).
		"domainName := self getTextDomainForPackage: (PackageOrganizer default packageOfMethod: methodReference)".
		domainName := 'Etoys-Tiles'.
		literalsForDomain := domains at: domainName ifAbsentPut: [Dictionary new].
		r fourth do: [ :literal |
			references := literalsForDomain at: literal ifAbsentPut: [OrderedCollection new].
			references add: methodReference.
		].
	]. 
	! !

!GetTextExporter methodsFor: 'private' stamp: 'bf 6/24/2012 22:22'!
createExtraInformation
	| extras |
	extras := OrderedCollection new.
	#(
		'ATTENTION TRANSLATORS!! This should be the name of your language as you would like it to appear in the Languages menu, e.g. "Español" or "English"' 'Language-Name'
		'ATTENTION TRANSLATORS!! Put in the directionality of your language, that is "LTR" for left-to-right or "RTL" for right-to-left' 'Language-Direction'
		) pairsDo: [:first :second |
			extras add: (Array with: '' with: first with: second).
	].
	^ extras! !

!GetTextExporter methodsFor: 'private-headers' stamp: 'tak 11/7/2006 22:53'!
createHeaders
	| headers |
	headers := OrderedCollection new.
	headers add: 'Project-Id-Version' -> 'eToys'.
	headers add: 'POT-Creation-Date' -> self currentDateAndTime.
	headers add: 'PO-Revision-Date' -> self currentDateAndTime.
	headers add: 'Last-Translator' -> ''.
	headers add: 'Language-Team' -> ''.
	headers add: 'MIME-Version' -> '1.0'.
	headers add: 'Content-Type' -> ('text/plain; charset=', stream converter class encodingNames first).
	headers add: 'Content-Transfer-Encoding' -> '8bit'.
	^ headers! !

!GetTextExporter methodsFor: 'private' stamp: 'KR 7/11/2007 13:03'!
currentDateAndTime
	^ String
		streamContents: [:aStream | 
			aStream nextPutAll: Date today yyyymmdd;
				space.
			Time now
				print24: true
				showSeconds: false
				on: aStream.
			aStream nextPutAll: '-0000']! !

!GetTextExporter methodsFor: 'exporting' stamp: 'bf 9/14/2007 19:30'!
dirNameCategory: category translator: translator 
	"Answer a file name for the category. Make one if it is not exist yet.
	Make template file name if translator is nil"
	"self new dirNameCategory: 'Morphic-Scripting Support' translator:
	NaturalLanguageTranslator current"
	"self new dirNameCategory: 'Morphic-Scripting Support' translator: nil"
	| safeCategory fileName dirName pathName |
	safeCategory := category copyReplaceAll: ' ' with: '_'.
	fileName := translator
				ifNil: [safeCategory , '.pot']
				ifNotNil: [translator localeID posixName , '.po'].
	dirName := (safeCategory findTokens: '-')
				inject: 'po'
				into: [:aString :next | aString , FileDirectory slash , next].
	pathName := dirName , FileDirectory slash , fileName.
	(FileDirectory default directoryNamed: dirName) assureExistence.
	^ pathName! !

!GetTextExporter methodsFor: 'exporting' stamp: 'KR 5/28/2010 01:16'!
dirNameDomain: domain translator: translator 
	"Answer a file name for the domain. Make one if it is not exist yet.
	Make template file name if translator is nil"
	"self new dirNameDomain: 'etoys' translator:
	NaturalLanguageTranslator current"
	"self new dirNameDomain: 'etoys' translator: nil"
	| fileName dirName pathName |
	"safeCategory := category copyReplaceAll: ' ' with: ':='."
	fileName := domain, 
						(translator
							ifNil: ['.pot']
							ifNotNil: ['.po']).
	dirName := 'po', FileDirectory slash, 
								(translator 
										ifNil: ['templates'] 
										ifNotNil: [translator localeID posixName]). 
	pathName := dirName , FileDirectory slash , fileName.
	(FileDirectory default directoryNamed: dirName) assureExistence.
	^ pathName! !

!GetTextExporter methodsFor: 'file out' stamp: 'bf 10/8/2007 13:26'!
exportBody: literals translator: translator 
	"Export a gettext file body. literals is a dictionary of keyword ->
	#(MethodReference...) in the textDomain."
	"Build {sortKey. comment. msgid } to optimize sorting (getting category is
	too slow).
	If there are two or more methods for a mgsid, only first method
	(alphabetical) is used for sorting."
	| sorted msgid sortedMethods category sortKey comment triplets commentUnderLined |
	triplets := literals associations
				collect: [:assoc | 
					msgid := assoc key.
					sortedMethods := assoc value asArray sort.
					category := (Smalltalk at: sortedMethods first classSymbol) category asString.
					sortKey := category , ',' , sortedMethods first printString , ',' , msgid.
					comment := (sortedMethods
								collect: [:each | each actualClass asString , '>>' , each methodSymbol asString])
								inject: category
								into: [:result :methodName | result , ',' , methodName].
					"Replace white spaces to _ because gettext tool might
					replace a space to a new line some times, and it makes
					difficult to take a diff."
					commentUnderLined := comment copyReplaceAll: ' ' with: '_'.
					Array
						with: sortKey
						with: commentUnderLined
						with: msgid].
	"Sort and output the words"
	sorted := triplets
				sort: [:a :b | a first <= b first].
	sorted
		do: [:triplet | 
			comment := triplet second.
			msgid := triplet third.
			self exportRecordHeader: comment.
			self
				exportPhrase: msgid
				translation: (self translationFor: msgid in: translator)]! !

!GetTextExporter methodsFor: 'private-headers' stamp: 'tak 11/8/2006 16:18'!
exportHeader
	self exportTag: 'msgid' msg: ''.
	self exportTag: 'msgstr' msg: ''.
	self createHeaders
		do: [:each | self exportHeaderLineKey: each key value: each value].
	stream cr; cr! !

!GetTextExporter methodsFor: 'private' stamp: 'KR 7/11/2007 13:02'!
exportHeaderLineKey: keyString value: valueString 
	stream nextPut: $";
		 nextPutAll: keyString;
		 nextPut: $:;
		 space;
		 nextPutAll: valueString;
		 nextPutAll: '\n';
		 nextPut: $";
		 cr.! !

!GetTextExporter methodsFor: 'private' stamp: 'KR 9/28/2007 13:15'!
exportHeader: domainName
	| headers |
	self exportTag: 'msgid' msg: ''.
	self exportTag: 'msgstr' msg: ''.
	headers := self createHeaders.
	headers add: 'X-Etoys-Domain' -> domainName.
	headers do: [:each | self exportHeaderLineKey: each key value: each value].
	stream cr; cr! !

!GetTextExporter methodsFor: 'private' stamp: 'yo 8/27/2008 14:25'!
exportInformation: anOrderedCollection
	anOrderedCollection do: [:each |
		self exportRecordHeader: each second.
		self exportPhrase: each third translation: ''].
	stream cr.! !

!GetTextExporter methodsFor: 'private' stamp: 'tak 10/2/2007 21:25'!
exportPhrase: phraseString translation: translationString 
	| normalizedTrans tmp transStartsWithCR transEndsWithCR|
	phraseString isEmpty
		ifTrue: [^ self].
	self exportTag: 'msgid' msg: phraseString.
	translationString size = 0 ifTrue: [
		normalizedTrans := ''
	] ifFalse: [
		transEndsWithCR := translationString last = (Character cr).
		phraseString last = (Character cr) ifTrue: [
			transEndsWithCR ifTrue: [
				normalizedTrans := translationString
			] ifFalse: [
				normalizedTrans :=  translationString , String cr
			]
		] ifFalse: [
			transEndsWithCR ifTrue: [
				normalizedTrans := translationString allButLast
			] ifFalse: [
				normalizedTrans := translationString
			]
		].
		transStartsWithCR := normalizedTrans first = (Character cr).
		phraseString first = (Character cr) ifTrue: [
			transStartsWithCR ifFalse: [
				tmp := (Character cr asString) , normalizedTrans.
				normalizedTrans := tmp.
			]
		] ifFalse: [
			transStartsWithCR ifTrue: [
				normalizedTrans := normalizedTrans allButFirst
			]
		]
	].
	self exportTag: 'msgstr' msg: normalizedTrans.
	stream cr! !

!GetTextExporter methodsFor: 'private' stamp: 'KR 7/13/2007 13:19'!
exportRecordHeader: context
	stream 
		nextPutAll: '#: ';
		nextPutAll: context;
		cr.! !

!GetTextExporter methodsFor: 'private' stamp: 'nice 3/31/2012 15:40'!
exportTag: tag msg: aString 
	stream nextPutAll: tag.
	stream space.
	aString lineIndicesDo: [:start :endWithoutDelimiters :end |
		| line |
		line := (end = endWithoutDelimiters)
			ifTrue: [aString copyFrom: start to: endWithoutDelimiters]
			ifFalse: [(aString at: endWithoutDelimiters + 1) = Character cr
				ifTrue: [aString copyFrom: start to: endWithoutDelimiters + 1]
				ifFalse: [(aString copyFrom: start to: endWithoutDelimiters) copyWith: Character cr]].
		stream
			nextPut: $";
			nextPutAll: (self formatString: line);
			nextPut: $";
			cr].! !

!GetTextExporter methodsFor: 'exporting' stamp: 'tak 7/23/2007 00:39'!
exportTemplate
	"GetTextExporter2 new exportTemplate"
	self exportTranslator: nil! !

!GetTextExporter methodsFor: 'exporting' stamp: 'KR 9/28/2007 13:14'!
exportTranslator: translator 
	"Export translation files. the file extention is 'po', or 'pot' if translator is nil "
	"GetTextExporter2 new exportTranslator: NaturalLanguageTranslator current "
	| domains |
	domains := Dictionary new.
	self appendTranslations: domains.
	domains
		keysAndValuesDo: [:domainName :value |
			self
				export: value
				translator: translator
				domain: domainName]! !

!GetTextExporter methodsFor: 'private' stamp: 'Richo 5/25/2010 22:06'!
export: literals translator: translator domain: domainName 
	| fileName |
	"Export a gettext file in a category. literals is a dictionary of keyword -> #(MethodReference...) in the textDomain."
	fileName := self dirNameDomain: domainName translator: translator.
	[stream := FileStream forceNewFileNamed: fileName.
	stream lineEndConvention: #lf.
	stream converter: UTF8TextConverter new.
	self exportHeader: domainName.
	domainName = TextDomainManager defaultDomain
	ifTrue: [self exportInformation: self createExtraInformation].
	self exportBody: literals translator: translator]
		ensure: [stream close]! !

!GetTextExporter methodsFor: 'private' stamp: 'KR 7/11/2007 13:07'!
formatReplacements
	| replacements |
	replacements := OrderedCollection new.
	replacements add: '\' -> '\\'.
	replacements add: String cr -> '\n'.
	replacements add: String tab -> '\t'.
	replacements add: '"' -> '\"'.
	^ replacements! !

!GetTextExporter methodsFor: 'private' stamp: 'KR 7/11/2007 13:08'!
formatString: aString 
	| result |
	result := aString.
	self formatReplacements
		do: [:each | result := result copyReplaceAll: each key with: each value].
	^ result! !

!GetTextExporter methodsFor: 'as yet unclassified' stamp: 'Richo 5/6/2010 17:55'!
getTextDomainForPackage: aPackageInfo 
	^TextDomainManager domainForPackage: aPackageInfo! !

!GetTextExporter methodsFor: 'accessing' stamp: 'tak 10/2/2007 15:53'!
stream
	^ stream! !

!GetTextExporter methodsFor: 'accessing' stamp: 'tak 10/2/2007 15:53'!
stream: aStream
	stream := aStream! !

!GetTextExporter methodsFor: 'private' stamp: 'Richo 4/14/2011 17:22'!
translationFor: aKey in: translator
	| translation |
	translator ifNil: [^ ''].
	TextDomainManager allKnownDomains do: [:domain |
		translation := translator translate: aKey inDomain: domain.
		aKey = translation ifFalse: [^translation]
	].
	^ aKey! !

Object subclass: #GetTextInterchange
	instanceVariableNames: 'language stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GetText-Editor'!

GetTextInterchange subclass: #GetTextImporter
	instanceVariableNames: 'msgId msgStr state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GetText-Editor'!

!GetTextImporter commentStamp: 'tak 10/24/2007 11:23' prior: 0!
GetTextImporter load gettext po file into a InternalTranslator.

GetTextImporter new importID: (LocaleID isoString: 'lang-name') fileNamed: 'lang.po'
!

!GetTextImporter class methodsFor: 'utilities' stamp: 'tak 10/18/2007 21:14'!
cleanUpUnnecessaryPhrases
	| keys refuse replaceBlock reader writer char result |
	"GetTextImporter cleanUpUnnecessaryPhrases"
	""
	"Collect wrong phrases"
	keys := InternalTranslator allKnownPhrases copy keys.
	refuse := Set new.
	"replaceBlock value: 'te\\nst'."
	replaceBlock := [:aString | 
			reader := aString readStream.
			writer := '' writeStream.
			[reader atEnd]
				whileFalse: [char := reader next.
					(char = $\
							and: [reader peek = $\])
						ifFalse: [writer nextPut: char]].
			writer contents].
	keys
		do: [:each | 
			result := replaceBlock value: each.
			(result ~= each
					and: [keys includes: result])
				ifTrue: [refuse add: each].
			result := GetTextImporter new formatString: each.
			(result ~= each
					and: [keys includes: result])
				ifTrue: [refuse add: each]].
	""
	"Remove from translated"
	InternalTranslator cachedTranslations
		do: [:each | refuse
				do: [:key | each translations
						removeKey: key
						ifAbsent: []]].
	""
	"Remove from untranslated"
	refuse
		do: [:key | InternalTranslator allKnownPhrases
				removeKey: key
				ifAbsent: []]! !

!GetTextImporter class methodsFor: 'utilities' stamp: 'tak 10/18/2007 21:14'!
importAll
	"GetTextImporter importAll"
	"Import all gettext files on po/. Only registered language is imported"
	InternalTranslator cachedTranslations
		do: [:translator | self import: translator allDirectory: FileDirectory default]! !

!GetTextImporter class methodsFor: 'utilities' stamp: 'tak 8/6/2007 12:12'!
import: translator allDirectory: aFileDirectory 
	"self import: NaturalLanguageTranslator current allDirectory:
	FileDirectory default"
	| fileName targetFile |
	fileName := translator localeID posixName , '.po'.
	(FileDirectory default directoryNamed: 'po')
		withAllSubdirectoriesCollect: [:each | (each fileExists: fileName)
				ifTrue: [targetFile := each fullNameFor: fileName.
					self new import: translator fileNamed: targetFile]]! !

!GetTextImporter methodsFor: 'parsing' stamp: 'tak 11/7/2006 00:28'!
appendId: aString 
	msgId := msgId , aString! !

!GetTextImporter methodsFor: 'parsing' stamp: 'tak 11/7/2006 00:28'!
appendStr: aString 
	msgStr := msgStr , aString! !

!GetTextImporter methodsFor: 'private' stamp: 'tak 11/8/2006 14:33'!
formatString: aString 
"
	self assert: (GetTextImporter new formatString: 'test') = 'test'.
	self assert: (GetTextImporter new formatString: 'te\nst') = ('te', String cr, 'st').
	self assert: (GetTextImporter new formatString: 'te\\nst') = ('te\nst').
	self assert: (GetTextImporter new formatString: 'te\\st') = ('te\st').
	self assert: (GetTextImporter new formatString: 'te\st') = ('te\st').
"
	| reader writer char |
	reader := aString readStream.
	writer := '' writeStream.
	[reader atEnd]
		whileFalse: [char := reader next.
			(char = $\
					and: [reader atEnd not])
				ifTrue: [char := reader next.
					char caseOf: {
						[$n] -> [writer nextPut: Character cr].
						[$t] -> [writer nextPut: Character tab].
						[$"] -> [writer nextPut: $"].
						[$\] -> [writer nextPut: $\]}
						 otherwise: [writer nextPutAll: {$\. char}]]
				ifFalse: [writer nextPut: char]].
	^ writer contents! !

!GetTextImporter methodsFor: 'importing' stamp: 'tak 10/24/2007 12:46'!
importID: localeID fileNamed: fileName
	| currentPlatform |
	language := InternalTranslator newLocaleID: localeID.
	currentPlatform := Locale currentPlatform.
	[Locale
		currentPlatform: (Locale localeID: localeID).
	[stream := FileStream readOnlyFileNamed: fileName.
	stream text.
	self parse]
		ensure: [stream notNil
				ifTrue: [stream close]]]
		ensure: [Locale currentPlatform: currentPlatform].
	NaturalLanguageTranslator privateStartUp "Actually it is not private no more...".! !

!GetTextImporter methodsFor: 'importing' stamp: 'tak 9/27/2007 12:09'!
import: aLanguage 
	^ self import: aLanguage fileNamed:  aLanguage localeID posixName , '.po'! !

!GetTextImporter methodsFor: 'importing' stamp: 'tak 10/24/2007 11:03'!
import: aLanguage fileNamed: fileName
	self importID: aLanguage localeID fileNamed: fileName! !

!GetTextImporter methodsFor: 'initialize-release' stamp: 'tak 11/7/2006 00:32'!
initialize
	msgId := ''.
	msgStr := ''.
	state := nil! !

!GetTextImporter methodsFor: 'parsing' stamp: 'tak 8/6/2007 12:23'!
parse
	| size |
	size := (stream isKindOf: FileStream)
				ifTrue: [stream size]
				ifFalse: [1].
	ProgressInitiationException
		display: 'Importing phrases from a gettext file.'
		during: [:bar | [stream atEnd]
				whileFalse: [| line | 
					line := stream upTo: Character linefeed.
					self
						parseLine: ((line endsWith: String cr)
								ifTrue: [line allButLast]
								ifFalse: [line]).
					bar value: stream position / size]].
	self storeTranslation! !

!GetTextImporter methodsFor: 'parsing' stamp: 'tak 7/22/2007 23:08'!
parseLine: lineString 
	(lineString beginsWith: '"Content-Type:')
		ifTrue: [self setContentType: lineString.
			^ self].
	(lineString beginsWith: '#')
		ifTrue: ["do nothing"
			^ self].
	lineString = ''
		ifTrue: [^ self storeTranslation].
	(lineString beginsWith: 'msgid')
		ifTrue: [state := #appendId:.
			self parseMsg: lineString.
			^ self].
	(lineString beginsWith: 'msgstr')
		ifTrue: [state := #appendStr:.
			self parseMsg: lineString.
			^ self].
	self parseMsg: lineString! !

!GetTextImporter methodsFor: 'parsing' stamp: 'tak 11/8/2006 14:42'!
parseMsg: lineString 
	| begin end msg |
	begin := lineString indexOf: $".
	end := lineString lastIndexOf: $".
	msg := begin + 1 <= (end - 1)
				ifTrue: [lineString copyFrom: begin + 1 to: end - 1]
				ifFalse: [''].
	state
		ifNotNil: [self perform: state with: msg].
	^ msg! !

!GetTextImporter methodsFor: 'parsing' stamp: 'tak 3/22/2006 15:10'!
setContentType: lineString 
	"self new setContentType: 'Content-Type: text/plain; charset=utf-8'"
	| reader charSet |
	reader := lineString readStream.
	reader upTo: $=.
	charSet := reader upTo: $\.
	stream
		converter: (TextConverter newForEncoding: charSet)! !

!GetTextImporter methodsFor: 'parsing' stamp: 'tak 10/18/2007 21:15'!
storeTranslation
	| key |
	key := self formatString: msgId.
	msgId isEmpty
		ifFalse: [InternalTranslator registerPhrase: key.
			msgStr isEmpty
				ifFalse: [language
						rawPhrase: key
						translation: (self formatString: msgStr)]].
	self initialize! !

!GetTextInterchange methodsFor: 'private' stamp: 'tak 8/6/2007 12:12'!
defaultFileName
	^ language localeID posixName , '.po'! !

!GetTextInterchange methodsFor: 'accessing' stamp: 'tak 3/23/2006 13:52'!
language: translator
	language := translator! !

!GetTextInterchange methodsFor: 'accessing' stamp: 'tak 3/23/2006 13:53'!
stream
	^ stream! !

!GetTextInterchange methodsFor: 'accessing' stamp: 'tak 3/23/2006 13:52'!
stream: aStream
	stream := aStream! !

SystemWindow subclass: #LanguageEditor
	instanceVariableNames: 'translator translations untranslated selectedTranslation selectedTranslations selectedUntranslated translationsList untranslatedList translationText translationsFilter untranslatedFilter newerKeys'
	classVariableNames: 'CheckMethods'
	poolDictionaries: ''
	category: 'GetText-Editor'!

!LanguageEditor commentStamp: 'dgd 11/16/2003 15:02' prior: 0!
Editor for Babel's languages.

Open it from

	World Menu >> open... >> Language Editor			(to open on default language)
	World Menu >> open... >> Language Editor for...	(to choose the language)

Or click:

	LanguageEditor openOnDefault.
	LanguageEditor open.

See http://swiki.agro.uba.ar/small_land/191 for documentation
!

!LanguageEditor class methodsFor: 'private' stamp: 'mir 7/21/2004 18:47'!
checkMethods
	^CheckMethods ifNil: [CheckMethods := self initCheckMethods]! !

!LanguageEditor class methodsFor: 'private' stamp: 'dgd 11/9/2003 15:39'!
ensureVisibilityOfWindow: aWindow 
	"private - activate the window"
	| |
	aWindow expand.
	aWindow comeToFront.
	""
	aWindow
		right: (aWindow right min: World right).
	aWindow
		bottom: (aWindow bottom min: World bottom).
	aWindow
		left: (aWindow left max: World left).
	aWindow
		top: (aWindow top max: World top).
	""
	aWindow flash; flash! !

!LanguageEditor class methodsFor: 'initialize-release' stamp: 'mir 7/21/2004 18:47'!
initCheckMethods
	"LanguageEditor initCheckMethods"

	| registry |
	registry := Dictionary new.
	registry
		at: 'es' put: #checkSpanishPhrase:translation:;
		yourself.
	^registry! !

!LanguageEditor class methodsFor: 'initialize-release' stamp: 'mt 5/10/2015 12:39'!
initialize
	"initialize the receiver"
	"(TheWorldMenu respondsTo: #registerOpenCommand:)
		ifTrue: [
			TheWorldMenu registerOpenCommand: {'Language Editor' translated. {self. #openOnDefault}}.
			TheWorldMenu registerOpenCommand: {'Language Editor for...' translated. {self. #open}}]"! !

!LanguageEditor class methodsFor: 'instance creation' stamp: 'kfr 4/26/2015 08:28'!
on: localeID 
	"answer an instance of the receiver on aLanguage"
	^ self new
		initializeOn: (InternalTranslator cachedTranslations
				at: localeID
				ifAbsent: [self
						inform: ('Translator for {1} is not found' translated format: {localeID}). ^nil])! !

!LanguageEditor class methodsFor: 'opening' stamp: 'kfr 4/26/2015 08:23'!
open
	"open the receiver on any language"
	" 
	LanguageEditor open. 
	"
	| menu availableLanguages |
	menu := MenuMorph new defaultTarget: self.
	menu addTitle: 'Language Editor for...' translated.
	""
	availableLanguages := (InternalTranslator availableLanguageLocaleIDs asSortedCollection: [:x :y | x asString <= y asString]).
	availableLanguages ifEmpty:[^self inform:'InternalTranslator not initialized'].
	availableLanguages
		do: [:eachLanguage | ""
			menu
				add: eachLanguage name
				target: self
				selector: #openOn:
				argument: eachLanguage].
	""
	menu popUpInWorld! !

!LanguageEditor class methodsFor: 'opening' stamp: 'TN 4/8/2005 01:13'!
openOnDefault
	"open the receiver on the default language"
	self openOn: LocaleID current! !

!LanguageEditor class methodsFor: 'instance creation' stamp: 'kfr 4/26/2015 08:29'!
openOn: aLanguage 
	"open an instance on aLanguage"
	| editor |
	World submorphs
		do: [:each | ""
			((each isKindOf: LanguageEditor)
					and: [each translator == aLanguage])
				ifTrue: [""
					self ensureVisibilityOfWindow: each.
					^ self]].
	""
	 editor := self on: aLanguage.
	 editor ifNotNil:[^editor openInWorld]! !

!LanguageEditor class methodsFor: 'initialize-release' stamp: 'dgd 11/9/2003 14:27'!
unload
	"the receiver is being unloaded"
	(TheWorldMenu respondsTo: #registerOpenCommand:)
		ifTrue: [""
			TheWorldMenu unregisterOpenCommand: 'Language Editor'.
			TheWorldMenu unregisterOpenCommand: 'Language Editor for...'] ! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'fbs 5/23/2013 08:45'!
addTranslation
	"translate a phrase"
	| phrase |
	phrase := UIManager default
				request: 'enter the original:' translated
				initialAnswer: ''.

	(phrase isNil
			or: [phrase = ''])
		ifTrue: [
			self beep.
			^ self].

	self translatePhrase: phrase! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'mir 7/21/2004 16:55'!
applyTranslations
	"private - try to apply the translations as much as possible all  
	over the image"
	Project current updateLocaleDependents! !

!LanguageEditor methodsFor: 'reporting' stamp: 'mir 7/21/2004 19:24'!
asHtml: aString 
	| stream |
	stream := String new writeStream.

	aString
		do: [:each | 
			each caseOf: {
				[Character cr] -> [stream nextPutAll: '<br>'].
				[$&] -> [stream nextPutAll: '&amp;'].
				[$<] -> [stream nextPutAll: '&lt;'].
				[$>] -> [stream nextPutAll: '&gt;'].
				[$*] -> [stream nextPutAll: '&star;'].
				[$@] -> [stream nextPutAll: '&at;']}
				 otherwise: [stream nextPut: each]].

	^ stream contents! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'yo 7/30/2004 22:25'!
browseMethodsWithTranslation
	| translation |
	self selectedTranslation isZero
		ifTrue: [""
			self beep.
			self inform: 'select the translation to look for' translated.
			^ self].
	""
	translation := self translations at: self selectedTranslation.
	self systemNavigation browseMethodsWithLiteral: translation! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:17'!
browseMethodsWithUntranslated
	| untrans |
	self selectedUntranslated isZero
		ifTrue: [""
			self beep.
			self inform: 'select the untrans phrase to look for' translated.
			^ self].
	""
	untrans := self untranslated at: self selectedUntranslated.
	SystemNavigation default browseMethodsWithLiteral: untrans.
! !

!LanguageEditor methodsFor: 'private' stamp: 'nice 12/25/2013 22:06'!
check
	"check the translations and answer a collection with the results"
	| results counter phrasesCount checkMethod |
	results := OrderedCollection new.
	untranslated := self untranslated.
	phrasesCount := self translations size + self untranslated size.
	counter := 0.
	checkMethod := self class checkMethods at: self translator localeID printString ifAbsent: [^results].
	
	self translations
		keysAndValuesDo: [:phrase :translation | 
			| result | 
			result := self perform: checkMethod with: phrase with: translation.
			(result notNil
					and: [result notEmpty])
				ifTrue: [results add: {phrase. translation. result}].
		
			counter := counter + 1.
			(counter isDivisibleBy: 50)
				ifTrue: [| percent | 
					percent := counter / phrasesCount * 100 printShowingMaxDecimalPlaces: 2.
					Transcript
						show: ('- checked {1} phrases of {2} ({3}%)...' translated format: {counter. phrasesCount. percent});
						 cr]].

	self untranslated
		do: [:phrase | 
			| result | 
			result := self checkUntranslatedPhrase: phrase.
			(result notNil
					and: [result notEmpty])
				ifTrue: [results add: {phrase. nil. result}].
		
			counter := counter + 1.
			(counter isDivisibleBy: 50)
				ifTrue: [| percent | 
					percent := counter / phrasesCount * 100 printShowingMaxDecimalPlaces: 2.
					Transcript
						show: ('- checked {1} phrases of {2} ({3}%)...' translated format: {counter. phrasesCount. percent});
						 cr]].

	^ results! !

!LanguageEditor methodsFor: 'private' stamp: 'mir 7/21/2004 18:58'!
checkPhrase: phraseString translation: translationString
	^nil! !

!LanguageEditor methodsFor: 'private' stamp: 'yo 12/1/2007 14:11'!
checkSpanishPhrase: phraseString translation: translationString 
	"check the translation and aswer a string with a comment or a 
	nil meaning no-comments"
	| superResult |
	superResult := self checkPhrase: phraseString translation: translationString.
	superResult isNil
		ifFalse: [^ superResult].
	"For some reason, MCInstaller couldn't read Spanish character. "
	"((translationString includes: $?)
	and: [(translationString includes: $ø) not]) 
		ifTrue: [^ 'øOlvidÛ el signo de pregunta?'].
	((translationString includes: $!!)
	and: [(translationString includes: $°) not])
		ifTrue: [^ 'øOlvidÛ el signo de admiraciÛn?'].
	"
	^ nil
! !

!LanguageEditor methodsFor: 'private' stamp: 'mir 7/21/2004 18:57'!
checkUntranslatedPhrase: phraseString 
	"check the phrase an aswer a string with a comment or a nil  
	meaning no-comments"

	(self translations includes: phraseString)
		ifTrue: [^ 'possible double-translation' translated].

	^ nil! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'KR 5/10/2008 15:31'!
codeSelectedTranslation
	| keys code |
	keys := selectedTranslations
				collect: [:key | self translations at: key].
	code := String
				streamContents: [:aStream | self translator fileOutOn: aStream keys: keys withBOM: false].
	(StringHolder new contents: code)
		openLabel: 'exported codes' translated! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'nice 5/1/2015 01:37'!
codeSelectedTranslationAsMimeString
	| keys code cont |
	keys := selectedTranslations
				collect: [:key | self translations at: key].
	code := String
				streamContents: [:aStream | self translator fileOutOn: aStream keys: keys withBOM: false].

	cont := String streamContents: [:strm |
		strm nextPutAll: '"UTF8+Gzip+Base64 encoded translation for;'; cr.
		strm nextPutAll: '#('.
		keys do: [:each | strm  nextPutAll: '''', each, ''' '.].
		strm nextPutAll: ')"'; cr; cr.
		strm nextPutAll: 'NaturalLanguageTranslator loadForLocaleIsoString: '.
		strm nextPut: $'.
		strm nextPutAll: translator localeID isoString.
		strm nextPut: $'.
		strm nextPutAll: ' fromGzippedMimeLiteral: '.
		strm nextPut: $'.
		strm nextPutAll: code squeakToUtf8 asByteArray zipped base64Encoded.
		strm nextPutAll: '''.'.
		strm cr.
	].
	
	(StringHolder new contents: cont)
		openLabel: 'exported codes in UTF8+Gzip+Base64 encoding' translated! !

!LanguageEditor methodsFor: 'initialization - toolbar' stamp: 'tak 11/16/2004 15:07'!
createButtonLabel: aString action: actionSelector help: helpString 
	"create a toolbar for the receiver"
	| button |
	button := SimpleButtonMorph new target: self;
				 label: aString translated "font: Preferences standardButtonFont";
				 actionSelector: actionSelector;
				 setBalloonText: helpString translated;
				 color: translator defaultBackgroundColor twiceDarker;
				 borderWidth: 2;
				 borderColor: #raised.
	""
	^ button! !

!LanguageEditor methodsFor: 'initialization - toolbar' stamp: 'AB 3/2/2006 15:55'!
createMainToolbar
	"create a toolbar for the receiver"
	| toolbar |
	toolbar := self createRow.
	""
"	toolbar
		addMorphBack: (self
				createUpdatingButtonWording: #debugWording
				action: #switchDebug
				help: 'Switch the debug flag')."
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	""
	toolbar
		addMorphBack: (self
				createButtonLabel: 'new'
				action: #newTranslations
				help: 'Create translations for new language.').
	toolbar
		addMorphBack: (self
				createButtonLabel: 'save'
				action: #saveToFile
				help: 'Save the translations to a file').
	toolbar
		addMorphBack: (self
				createButtonLabel: 'load'
				action: #loadFromFile
				help: 'Load the translations from a file').
	toolbar
		addMorphBack: (self
				createButtonLabel: 'merge'
				action: #mergeFromFile
				help: 'Merge the current translations with the translations in a file').
	""
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	toolbar
		addMorphBack: (self
				createButtonLabel: 'apply'
				action: #applyTranslations
				help: 'Apply the translations as much as possible.').
	""
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	toolbar
		addMorphBack: (self
				createButtonLabel: 'check translations'
				action: #check
				help: 'Check the translations and report the results.').
	toolbar
		addMorphBack: (self
				createButtonLabel: 'report'
				action: #report
				help: 'Create a report.').
	toolbar
		addMorphBack: (self
				createButtonLabel: 'gettext'
				action: #getText
				help: 'Interface with gettext.').
	""
	^ toolbar! !

!LanguageEditor methodsFor: 'initialization - toolbar' stamp: 'dgd 9/21/2003 11:46'!
createRow
	"create a row"
	| row |
	row := AlignmentMorph newRow.
	row layoutInset: 3;
		 wrapCentering: #center;
		 cellPositioning: #leftCenter.
	""
	^ row! !

!LanguageEditor methodsFor: 'initialization - statusbar' stamp: 'tak 11/15/2004 12:15'!
createStatusbar
	"create the statusbar for the receiver"
	| statusbar |
	statusbar := self createRow.
	statusbar addMorph: ((UpdatingStringMorph on: self selector: #status) growable: true;
			 useStringFormat;
			 hResizing: #spaceFill;
			 stepTime: 2000).
	^ statusbar! !

!LanguageEditor methodsFor: 'initialization - toolbar' stamp: 'yo 2/17/2005 18:24'!
createTranslationsToolbar
	"create a toolbar for the receiver"
	| toolbar |
	toolbar := self createRow.
	""
	toolbar
		addMorphBack: (self
				createUpdatingButtonWording: #translationsFilterWording
				action: #filterTranslations
				help: 'Filter the translations list.').
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	""
	toolbar
		addMorphBack: (self
				createButtonLabel: 'search'
				action: #searchTranslation
				help: 'Search for a translation containing...').
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	toolbar
		addMorphBack: (self
				createButtonLabel: 'remove'
				action: #removeTranslation
				help: 'Remove the selected translation.  If none is selected, ask for the one to remove.').
	""
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	toolbar
		addMorphBack: (self
				createButtonLabel: 'where'
				action: #browseMethodsWithTranslation
				help: 'Launch a browser on all methods that contain the phrase as a substring of any literal String.').
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	toolbar
		addMorphBack: (self
				createButtonLabel: 'r-unused'
				action: #removeTranslatedButUnusedStrings
				help: 'Remove all the strings that are not used by the system').
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	toolbar
		addMorphBack: (self
				createButtonLabel: 'add '
				action: #addTranslation
				help: 'Add a new phrase').

	^ toolbar! !

!LanguageEditor methodsFor: 'initialization - toolbar' stamp: 'sd 12/18/2004 18:10'!
createUntranslatedToolbar
	"create a toolbar for the receiver"
	| toolbar |
	toolbar := self createRow.
	""
	toolbar
		addMorphBack: (self
				createUpdatingButtonWording: #untranslatedFilterWording
				action: #filterUntranslated
				help: 'Filter the untranslated list.').
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	""
	toolbar
		addMorphBack: (self
				createButtonLabel: 'search'
				action: #searchUntranslated
				help: 'Search for a untranslated phrase containing...').
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	toolbar
		addMorphBack: (self
				createButtonLabel: 'remove'
				action: #removeUntranslated
				help: 'Remove the selected untranslated phrease.  If none is selected, ask for the one to remove.').
	""
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	toolbar
		addMorphBack: (self
				createButtonLabel: 'translate'
				action: #translate
				help: 'Translate the selected untranslated phrase or a new phrase').
	""
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	toolbar
		addMorphBack: (self
				createButtonLabel: 'where'
				action: #browseMethodsWithUntranslated
				help: 'Launch a browser on all methods that contain the phrase as a substring of any literal String.').
	toolbar addTransparentSpacerOfSize: 5 @ 0.
	toolbar
		addMorphBack: (self
				createButtonLabel: 'r-unused'
				action: #removeUntranslatedButUnusedStrings
				help: 'Remove all the strings that are not used by the system').
	^ toolbar! !

!LanguageEditor methodsFor: 'initialization - toolbar' stamp: 'tak 11/16/2004 15:06'!
createUpdatingButtonWording: wordingSelector action: actionSelector help: helpString 
	"create a toolbar for the receiver"
	| button |
	button := (UpdatingSimpleButtonMorph newWithLabel: '-') target: self;
				 wordingSelector: wordingSelector;
				 actionSelector: actionSelector;
				 setBalloonText: helpString translated;
				 color: translator defaultBackgroundColor twiceDarker;
				 borderWidth: 1;
				 borderColor: #raised; cornerStyle: #square.
	""
	^ button! !

!LanguageEditor methodsFor: 'open/close' stamp: 'dgd 8/26/2003 14:20'!
delete
	"Remove the receiver as a submorph of its owner"
	self model: nil.
	super delete ! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 11/28/2004 14:27'!
deselectAllTranslation
	selectedTranslations := IdentitySet new.
	self changed: #allSelections! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'fbs 5/23/2013 08:45'!
filterTranslations
	| filter |
	filter := UIManager default
		request: 'filter with
(empty string means no-filtering)' translated
		initialAnswer: self translationsFilter.

	self filterTranslations: filter! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:36'!
filterTranslations: aString 
| filter |
filter := aString ifNil:[''].
""
	translationsFilter := filter.
	self refreshTranslations.
! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'fbs 5/23/2013 08:45'!
filterUntranslated
	| filter |
	filter := UIManager default
		request: 'filter with
(empty string means no-filtering)' translated
		initialAnswer: self untranslatedFilter.

	self filterUntranslated: filter! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:30'!
filterUntranslated: aString 
	| filter |
	filter := aString
				ifNil: [''].
	""
	untranslatedFilter := filter.
	self refreshUntranslated! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 7/23/2007 00:38'!
getText
	| menu |
	menu := MenuMorph new defaultTarget: self.
	""
	menu
		add: 'gettext template' translated
		target: self
		selector: #getTextExportTemplate.
	menu lastItem setBalloonText: 'exports the translations to gettext separated format.' translated.
	menu
		add: 'gettext export' translated
		target: self
		selector: #getTextExport.
	menu lastItem setBalloonText: 'Exports the translations to GetText format.' translated.

	""
	menu
		add: 'gettext import' translated
		target: self
		selector: #getTextImport.
	menu lastItem setBalloonText: 'Imports the translations from GetText format.' translated.
	""
	menu popUpInWorld! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'edc 3/29/2012 06:54'!
getTextExport
	Cursor wait
		showWhile: [GetTextExporter new exportTranslator: self model]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'edc 3/29/2012 06:54'!
getTextExportTemplate
	Cursor wait
		showWhile: [GetTextExporter new exportTemplate] ! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'KR 5/10/2008 15:11'!
getTextImport
	| menu |
	menu := MenuMorph new defaultTarget: self.
	menu addTitle: 'Choose translation file' translated.
	menu add: 'All *.po files' translated action: #getTextImportAll.
	menu add: 'Choose a file' translated action: #getTextImportAFile.
	menu popUpInWorld! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 8/6/2007 11:13'!
getTextImportAFile
	| result |
	result := (StandardFileMenu new pattern: '*.po';
				
				oldFileFrom: (FileDirectory default directoryNamed: 'po')) startUpWithCaption: 'Select a File:' translated.
	result
		ifNil: [^ self].
	self
		withUnboundModelDo: [:trans | Cursor wait
				showWhile: [GetTextImporter new
						import: trans
						fileNamed: (result directory fullNameFor: result name)]]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 7/23/2007 01:12'!
getTextImportAll
	self
		withUnboundModelDo: [:trans | Cursor wait
				showWhile: [GetTextImporter import: trans allDirectory: FileDirectory default]].
	self refreshBoth! !

!LanguageEditor methodsFor: 'stef' stamp: 'ar 4/10/2005 18:48'!
identifyUnusedStrings
	"self new identifyUnusedStrings"
	translationsList getList
		do: [:each | 
			Transcript show: each.
			Transcript show: (Smalltalk
					allSelect: [:method | method
							hasLiteralSuchThat: [:lit | lit isString
									and: [lit includesSubstring: each caseSensitive: true]]]) size printString; cr]! !

!LanguageEditor methodsFor: 'initialization' stamp: 'yo 3/1/2005 12:33'!
initializeNewerKeys

	newerKeys := Set new.
! !

!LanguageEditor methodsFor: 'initialization' stamp: 'AB 3/2/2006 16:07'!
initializeOn: aLanguage 
	"initialize the receiver on aLanguage"
	""
	selectedTranslation := 0.
	selectedUntranslated := 0.
	selectedTranslations := IdentitySet new.
	""
	translator := aLanguage.
	""
	self model: aLanguage.
	self setLabel: 'Language editor for: ' translated , self translator name.
	""
	self initializeToolbars.
	self initializePanels.
	self initializeStatusbar.
	self initializeNewerKeys.
! !

!LanguageEditor methodsFor: 'initialization' stamp: 'pad 3/12/2006 10:45'!
initializePanels
	"initialize the receiver's panels"
	translationsList := PluggableListMorphOfMany
				on: self
				list: #translations
				primarySelection: #selectedTranslation
				changePrimarySelection: #selectedTranslation:
				listSelection: #selectedTranslationsAt:
				changeListSelection: #selectedTranslationsAt:put:
				menu: #translationsMenu:
				keystroke: #translationsKeystroke:.
	translationsList setBalloonText: 'List of all the translated phrases.' translated.
	""
	untranslatedList := PluggableListMorph
				on: self
				list: #untranslated
				selected: #selectedUntranslated
				changeSelected: #selectedUntranslated:
				menu: #untranslatedMenu:
				keystroke: #untranslatedKeystroke:.
	untranslatedList setBalloonText: 'List of all the untranslated phrases.' translated.
	""
	translationText := PluggableTextMorph
				on: self
				text: #translation
				accept: #translation:
				readSelection: nil
				menu: nil.
	translationText setBalloonText: 'Translation for the selected phrase in the upper list.' translated.
	""
	self
		addMorph: translationsList
		frame: (0 @ 0.18 corner: 0.5 @ 0.66).
	self
		addMorph: untranslatedList
		frame: (0.5 @ 0.18 corner: 1 @ 0.93).
	self
		addMorph: translationText
		frame: (0 @ 0.66 corner: 0.5 @ 0.93).
	self hResizing: #shrinkWrap! !

!LanguageEditor methodsFor: 'initialization - statusbar' stamp: 'dgd 9/21/2003 11:39'!
initializeStatusbar
	"initialize the receiver's statusbar"
	self
		addMorph: self createStatusbar
		frame: (0 @ 0.93 corner: 1 @ 1)! !

!LanguageEditor methodsFor: 'initialization - toolbar' stamp: 'dgd 9/21/2003 11:27'!
initializeToolbars
	"initialize the receiver's toolbar"
	self
		addMorph: self createMainToolbar
		frame: (0 @ 0 corner: 1 @ 0.09).
	""
	self
		addMorph: self createTranslationsToolbar
		frame: (0 @ 0.09 corner: 0.5 @ 0.18).
	self
		addMorph: self createUntranslatedToolbar
		frame: (0.5 @ 0.09 corner: 1 @ 0.18)! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:41'!
loadFromFile
	| fileName |
	fileName := self selectTranslationFileName.
	fileName isNil
		ifTrue: [""
			self beep.
			^ self].
	""
	Cursor wait
		showWhile: [
			self translator loadFromFileNamed: fileName.
			self refreshBoth]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:41'!
mergeFromFile
	| fileName |
	fileName := self selectTranslationFileName.
	fileName isNil
		ifTrue: [""
			self beep.
			^ self].
	""
	Cursor wait
		showWhile: [
			self translator loadFromFileNamed: fileName.
			self refreshBoth]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'fbs 5/23/2013 08:45'!
newTranslations
	"private - try to apply the translations as much as possible all 
	over the image"
	| result newID |
	result := UIManager default request: 'New locale ID string?' translated initialAnswer: Locale current determineLocaleID isoString.
	result isEmpty
		ifTrue: ["Do nothing"
			^ self].
	newID := LocaleID isoString: result.
	InternalTranslator
		newLocaleID: (LocaleID isoString: result).
	self class openOn: newID! !

!LanguageEditor methodsFor: 'stef' stamp: 'ar 4/10/2005 18:48'!
numberOfTimesStringIsUsed: aString

	^ (self systemNavigation allSelect: [:method | method
							hasLiteralSuchThat: [:lit | lit isString
									and: [lit includesSubstring: aString caseSensitive: true]]]) size! !

!LanguageEditor methodsFor: 'updating' stamp: 'dgd 8/28/2003 10:31'!
okToChange
	"Allows a controller to ask this of any model"
	self selectedTranslation isZero
		ifTrue: [^ true].
	""
	translationText hasUnacceptedEdits
		ifFalse: [^ true].
	^ (CustomMenu confirm: 'Discard the changes to currently selected translated phrase?' translated)
		and: [""
			translationText hasUnacceptedEdits: false.
			true]! !

!LanguageEditor methodsFor: 'message handling' stamp: 'gm 8/30/2003 01:54'!
perform: selector orSendTo: otherTarget 
	"I wish to intercept and handle selector myself"
	^ self perform: selector! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'fbs 5/23/2013 08:46'!
phraseToTranslate
	"answer a phrase to translate.  use the selected untranslated phrase or ask for a new one"
	^ self selectedUntranslated isZero
		ifTrue: [UIManager default
				multiLineRequest: 'new phrase to translate' translated
				centerAt: Sensor cursorPoint
				initialAnswer: ''
				answerHeight: 200]
		ifFalse: [self untranslated at: self selectedUntranslated]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:41'!
phrase: phraseString translation: translationString 
	"set the models's translation for phraseString"
	self translator phrase: phraseString translation: translationString.
	self refreshBoth.

	newerKeys add: phraseString.
! !

!LanguageEditor methodsFor: 'reporting' stamp: 'mir 8/11/2004 09:59'!
printHeaderReportOn: aStream 
	"append to aStream a header report of the receiver with swiki  
	format"
	aStream nextPutAll: '!!!!';
		
		nextPutAll: ('Language: {1}' translated format: {self translator localeID isoString});
		 cr.

	aStream nextPutAll: '- ';
		
		nextPutAll: ('{1} translated phrases' translated format: {self translator translations size});
		 cr.

	aStream nextPutAll: '- ';
		
		nextPutAll: ('{1} untranslated phrases' translated format: {self translator untranslated size});
		 cr.

	aStream cr; cr! !

!LanguageEditor methodsFor: 'reporting' stamp: 'mir 7/21/2004 19:25'!
printReportOn: aStream 
	"append to aStream a report of the receiver with swiki format"
	self printHeaderReportOn: aStream.
	self printUntranslatedReportOn: aStream.
	self printTranslationsReportOn: aStream! !

!LanguageEditor methodsFor: 'reporting' stamp: 'tak 10/19/2007 13:37'!
printTranslationsReportOn: aStream 
	"append to aStream a report of the receiver's translations"
	| originalPhrases |
	aStream nextPutAll: '!!';
		 nextPutAll: 'translations' translated;
		 cr.

	originalPhrases := self translator translations keys asSortedCollection.

	originalPhrases
		do: [:each | 
			aStream
				nextPutAll: ('|{1}|{2}|' format: {self asHtml: each. self
							asHtml: (self translator translate: each)});
				 cr].

	aStream cr; cr! !

!LanguageEditor methodsFor: 'reporting' stamp: 'mir 7/21/2004 19:26'!
printUntranslatedReportOn: aStream 
	"append to aStream a report of the receiver's translations"
	aStream nextPutAll: '!!';
		 nextPutAll: 'not translated' translated;
		 cr.

	self untranslated asSortedCollection
		do: [:each | 
			aStream
				nextPutAll: ('|{1}|' format: {self asHtml: each});
				 cr].

	aStream cr; cr! !

!LanguageEditor methodsFor: 'updating' stamp: 'tak 3/13/2006 20:40'!
refreshBoth
	self refreshUntranslated
! !

!LanguageEditor methodsFor: 'updating' stamp: 'tak 3/13/2006 20:40'!
refreshTranslations
	"refresh the translations panel"
	self selectedTranslation: 0.
	translations := nil.
	self changed: #translations.
! !

!LanguageEditor methodsFor: 'updating' stamp: 'tak 3/13/2006 20:39'!
refreshUntranslated
"refresh the untranslated panel"
	self refreshTranslations.
	self selectedUntranslated: 0.
	untranslated := nil.
	self changed: #untranslated.
! !

!LanguageEditor methodsFor: 'stef' stamp: 'tak 1/4/2005 09:26'!
removeTranslatedButUnusedStrings
	(self confirm: 'Are you sure that you want to remove unused strings?' translated)
		ifFalse: [^ self].
	translationsList getList
		do: [:each | 
			| timesUsed | 
			timesUsed := self numberOfTimesStringIsUsed: each.
			Transcript show: each.
			Transcript show: timesUsed printString;
				 cr.
			timesUsed isZero
				ifTrue: [self translator removeTranslationFor: each]]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:41'!
removeTranslation
	"remove the selected translation"
	| translation |
	self selectedTranslation isZero
		ifTrue: [""
			self beep.
			self inform: 'select the translation to remove' translated.
			^ self].
	""
	translation := self translations at: self selectedTranslation.
""
	(self
			confirm: ('Removing "{1}".
Are you sure you want to do this?' translated format: {translation}))
		ifFalse: [^ self].
""
	self translator removeTranslationFor: translation.
	self refreshBoth! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:18'!
removeUntranslated
	"remove the selected untranslated phrase"
	| untrans |
	self selectedUntranslated isZero
		ifTrue: [""
			self beep.
			self inform: 'select the untranslated phrase to remove' translated.
			^ self].
	""
	untrans := self untranslated at: self selectedUntranslated.
	""
	(self
			confirm: ('Removing "{1}".
Are you sure you want to do this?' translated format: {untrans}))
		ifFalse: [^ self].
	""
	self translator removeUntranslated: untrans! !

!LanguageEditor methodsFor: 'stef' stamp: 'tak 3/13/2006 20:30'!
removeUntranslatedButUnusedStrings
	(self confirm: 'Are you sure that you want to remove unused strings?' translated)
		ifFalse: [^ self].
	untranslatedList getList
		do: [:each | 
			| timesUsed | 
			timesUsed := self numberOfTimesStringIsUsed: each.
			Transcript show: each.
			Transcript show: timesUsed printString;
				 cr.
			timesUsed isZero 
				ifTrue: [self translator removeUntranslated: each]].

	self refreshUntranslated.
! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'mir 7/21/2004 19:27'!
report
	self reportString openInWorkspaceWithTitle: 'report' translated! !

!LanguageEditor methodsFor: 'reporting' stamp: 'mir 7/21/2004 19:26'!
reportString
	"answer a string with a report of the receiver"
	| stream |
	stream := String new writeStream.
	self printReportOn: stream.
	^ stream contents! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'yo 3/1/2005 12:36'!
resetNewerKeys

	self initializeNewerKeys.
! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'fbs 5/23/2013 08:46'!
saveToFile
	"save the translator to a file"
	| fileName |
	fileName := UIManager default request: 'file name' translated initialAnswer: translator localeID isoString , '.translation'.
	(fileName isNil
			or: [fileName isEmpty])
		ifTrue: [
			self beep.
			^ self].

Cursor wait
		showWhile: [
	self translator saveToFileNamed: fileName]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'fbs 5/23/2013 08:47'!
searchTranslation
	| search |
	search := UIManager default request: 'search for' translated initialAnswer: ''.
	(search isNil
			or: [search isEmpty])
		ifTrue: [
			self beep.
			^ self].

self searchTranslation: search! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 10/19/2007 13:37'!
searchTranslation: aString 
	| results index |
	results := self translations
				select: [:each | ""
					('*' , aString , '*' match: each)
						or: ['*' , aString , '*' match: (self translator translate: each)]].
	""
	results isEmpty
		ifTrue: [""
			self inform: 'no matches for' translated , ' ''' , aString , ''''.
			^ self].
	""
	results size = 1
		ifTrue: [""
			self selectTranslationPhrase: results first.
			^ self].
	""
	index := (PopUpMenu
				labelArray: (results
						collect: [:each | ""
							(each copy replaceAll: Character cr with: $\)
								, ' -> '
								, ((self translator translate: each) copy replaceAll: Character cr with: $\)]))
				startUpWithCaption: 'select the translation...' translated.
	""
	index isZero
		ifTrue: [""
			self beep.
			^ self].
	""
	self
		selectTranslationPhrase: (results at: index)! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'fbs 5/23/2013 08:47'!
searchUntranslated
	| search |
	search := UIManager default request: 'search for' translated initialAnswer: ''.
	(search isNil
			or: [search isEmpty])
		ifTrue: [
			self beep.
			^ self].

	self searchUntranslated: search! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'dgd 9/1/2003 14:28'!
searchUntranslated: aString 
	| untranslateds results index |
	untranslateds := self untranslated.
	results := untranslateds
				select: [:each | '*' , aString , '*' match: each].
	""
	results isEmpty
		ifTrue: [""
			self inform: 'no matches for' translated , ' ''' , aString , ''''.
			^ self].
	""
	results size = 1
		ifTrue: [""
			self selectUntranslatedPhrase: results first.
			^ self].
	""
	index := (PopUpMenu
				labelArray: (results
						collect: [:each | each copy replaceAll: Character cr with: $\]))
				startUpWithCaption: 'select the untranslated phrase...' translated.
	""
	index isZero
		ifTrue: [""
			self beep.
			^ self].
	""
	self
		selectUntranslatedPhrase: (results at: index)! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 11/28/2004 14:26'!
selectAllTranslation
	selectedTranslations := (1 to: self translations size) asIdentitySet.
	self changed: #allSelections! !

!LanguageEditor methodsFor: 'accessing' stamp: 'dgd 8/24/2003 19:13'!
selectedTranslation
	"answer the selectedTranslation"
	^ selectedTranslation! !

!LanguageEditor methodsFor: 'accessing' stamp: 'tak 11/28/2004 14:12'!
selectedTranslationsAt: index
	^ selectedTranslations includes: index! !

!LanguageEditor methodsFor: 'accessing' stamp: 'tak 11/28/2004 14:15'!
selectedTranslationsAt: index put: value 
	value = true
		ifTrue: [selectedTranslations add: index]
		ifFalse: [selectedTranslations
				remove: index
				ifAbsent: []]! !

!LanguageEditor methodsFor: 'accessing' stamp: 'dgd 8/24/2003 21:56'!
selectedTranslation: anInteger 
	"change the receiver's selectedTranslation"
	selectedTranslation := anInteger.
	""
	self changed: #selectedTranslation.
	self changed: #translation! !

!LanguageEditor methodsFor: 'accessing' stamp: 'dgd 8/24/2003 21:57'!
selectedUntranslated
	"answer the selectedUntranslated"
	^ selectedUntranslated! !

!LanguageEditor methodsFor: 'accessing' stamp: 'dgd 8/24/2003 21:57'!
selectedUntranslated: anInteger 
	"change the selectedUntranslated"
	selectedUntranslated := anInteger.
	""
	self changed: #selectedUntranslated! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:53'!
selectNewerKeys

	| index |
	self deselectAllTranslation.
	newerKeys do: [:k |
		index := self translations indexOf: k ifAbsent: [0].
		index > 0 ifTrue: [
			self selectedTranslationsAt: index put: true
		].
	].
! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'dgd 8/24/2003 22:15'!
selectTranslationFileName
	"answer a file with a translation"
	| file |
	file := (StandardFileMenu oldFileMenu: FileDirectory default withPattern: '*.translation')
				startUpWithCaption: 'Select the file...' translated.
	^ file isNil
		ifFalse: [file directory fullNameFor: file name]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'dgd 8/24/2003 22:49'!
selectTranslationPhrase: phraseString 
	self selectedTranslation: (self translations indexOf: phraseString)! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'dgd 8/27/2003 20:43'!
selectUntranslatedPhrase: phraseString 
	self
		selectedUntranslated: (self untranslated indexOf: phraseString)! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/22/2006 19:22'!
status
	"answer a status string"
	| translationsSize untranslatedSize |
	translationsSize := self translator translations size.
	untranslatedSize := self translator untranslated size.
	^ '| {1} phrases | {2} translated | {3} untranslated |' translated format: {translationsSize + untranslatedSize. translationsSize. untranslatedSize}! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:42'!
translate
	"translate a phrase"
	| phrase |
	phrase := self phraseToTranslate.
	""
	(phrase isNil
			or: [phrase = ''])
		ifTrue: [""
			self beep.
			^ self].
	""
	self translatePhrase: phrase.
	self refreshBoth! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'fbs 5/23/2013 08:48'!
translatePhrase: aString 
	"translate aString"
	| translation |
	translation := UIManager default
				multiLineRequest: 'translation for: ' translated , '''' , aString , ''''
				centerAt: Sensor cursorPoint
				initialAnswer: aString
				answerHeight: 200.

	(translation isNil
			or: [translation = ''])
		ifTrue: [""
			self beep.
			^ self].

	self phrase: aString translation: translation! !

!LanguageEditor methodsFor: 'accessing' stamp: 'tak 10/19/2007 13:37'!
translation
	"answer the translation for the selected phrase"
	self selectedTranslation isZero
		ifTrue: [^ '<select a phrase from the upper list>' translated].
	""
	^ self translator
		translate: (self translations at: self selectedTranslation)! !

!LanguageEditor methodsFor: 'accessing' stamp: 'tak 10/19/2007 13:37'!
translations
	"answet the translator's translations"
	| allTranslations filterString |
	translations ifNotNil: [^translations].
	allTranslations := self translator translations keys.
	""
	filterString := self translationsFilter.
	""
	filterString isEmpty
		ifFalse: [allTranslations := allTranslations
						select: [:each | ""
							('*' , filterString , '*' match: each)
								or: ['*' , filterString , '*'
										match: (self translator translate: each)]]].
""
	^ translations := allTranslations asSortedCollection asArray! !

!LanguageEditor methodsFor: 'accessing' stamp: 'dgd 9/21/2003 12:00'!
translationsFilter
^translationsFilter ifNil:['']! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'dgd 9/21/2003 12:09'!
translationsFilterWording
	^ (self translationsFilter isEmpty
		ifTrue: ['filter' translated]
		ifFalse: ['filtering: {1}' translated format:{self translationsFilter}]) ! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'gm 8/30/2003 02:00'!
translationsKeystroke: aChar 
	"Respond to a Command key in the translations list."
	aChar == $x
		ifTrue: [^ self removeTranslation].
	aChar == $E
		ifTrue: [^ self browseMethodsWithTranslation]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'yo 3/1/2005 12:49'!
translationsMenu: aMenu 
	^ aMenu add: 'remove (x)' translated action: #removeTranslation;
		 add: 'where (E)' translated action: #browseMethodsWithTranslation;
		 add: 'select all' translated action: #selectAllTranslation;
		 add: 'deselect all' translated action: #deselectAllTranslation;
		 add: 'select changed keys' translated action: #selectNewerKeys;
		 add: 'export selection' translated action: #codeSelectedTranslation;
		 add: 'export selection in do-it form' translated action: #codeSelectedTranslationAsMimeString;
		 add: 'reset changed keys' translated action: #resetNewerKeys;
		 yourself! !

!LanguageEditor methodsFor: 'accessing' stamp: 'yo 3/1/2005 12:44'!
translation: aStringOrText 
	"change the translation for the selected phrase"
	| phrase |
	self selectedTranslation isZero
		ifTrue: [^ self].
	phrase := self translations at: self selectedTranslation.
	translator
		phrase: phrase
		translation: aStringOrText asString.
	newerKeys add: phrase.
	^ true! !

!LanguageEditor methodsFor: 'private' stamp: 'mir 8/11/2004 09:57'!
translator
	^translator! !

!LanguageEditor methodsFor: 'accessing' stamp: 'tak 10/19/2007 13:37'!
untranslated
	"answer the translator's untranslated phrases"
	

	| all filterString |
	untranslated ifNotNil: [^ untranslated].
	all := self translator untranslated.
	""
	filterString := self untranslatedFilter.
	""
	filterString isEmpty
		ifFalse: [all := all
						select: [:each | ""
							('*' , filterString , '*' match: each)
								or: ['*' , filterString , '*'
										match: (self translator translate: each)]]].
	""
	^ untranslated := all asSortedCollection asArray! !

!LanguageEditor methodsFor: 'accessing' stamp: 'dgd 9/21/2003 12:19'!
untranslatedFilter
	^ untranslatedFilter
		ifNil: ['']! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'dgd 9/21/2003 12:19'!
untranslatedFilterWording
	^ self untranslatedFilter isEmpty
		ifTrue: ['filter' translated]
		ifFalse: ['filtering: {1}' translated format: {self untranslatedFilter}]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'gm 8/30/2003 02:01'!
untranslatedKeystroke: aChar 
	"Respond to a Command key in the translations list."
	aChar == $t
		ifTrue: [^ self translate].
	aChar == $E
		ifTrue: [^ self browseMethodsWithUntranslated]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'dgd 10/13/2003 18:30'!
untranslatedMenu: aMenu 
	^ aMenu add: 'remove' translated action: #removeUntranslated;
		 add: 'translate (t)' translated action: #translate;
		 add: 'where (E)' translated action: #browseMethodsWithUntranslated;
		 yourself! !

!LanguageEditor methodsFor: 'updating' stamp: 'dgd 8/25/2003 20:11'!
update: aSymbol 
	"Receive a change notice from an object of whom the receiver  
	is a dependent."
	super update: aSymbol.
	""
	aSymbol == #untranslated
		ifTrue: [self refreshUntranslated].
	aSymbol == #translations
		ifTrue: [self refreshTranslations]! !

!LanguageEditor methodsFor: 'private' stamp: 'dgd 8/13/2004 22:39'!
withUnboundModelDo: aBlock 
	"Private - Evaluate aBlock with the receiver temporary  
	unbound from the model.  
	 
	Useful to perform a batch of modifications to the model  
	without updating the view."
	| formerModel |
	formerModel := self model.
	self model: nil.
	[aBlock value: formerModel]
		ensure: [self model: formerModel]! !
LanguageEditor initialize!
